﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;

namespace API_Initialization
{
    class APILoader
    {
        //Переменная пути к дирректории с установленным T-FLEX CAD
        static private string pathProgramFolder;

        //Проверка на наличие дирректории с установленным T-FLEX CAD
        static public void Initialize(string subkey_name)
        {
            //Если дирректория с установленным T-FLEX CAD существует,
            //то работа метода завершается
            if (pathProgramFolder != null)
                return;

            //Инициализация переменной
            pathProgramFolder = "";

            //Генерация системного сообщения
            //Получение пути дирректории, куда установлена программа
            //например, C:\Program Files\T-FLEX\T-FLEX CAD 11\Program\
            string path = GetPath(subkey_name);

            //Если дирректория с установленным T-FLEX CAD не существует (значение null),
            if (string.IsNullOrEmpty(path))
                //то выдается системное сообщение
                throw new System.IO.FileNotFoundException("T-FLEX CAD not installed");

            //Переменной пути программной папки присваивается значение
            pathProgramFolder = path;

            //Обработка исключительной ситуации - когда разрешение сборки завершается неудачей
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(AssemblyResolve);
        }

        //Инициализация T-FLEX API
        static public bool InitializeTFlexCADAPI()
        {
            //Если дирректория с установленным T-FLEX CAD не существует (значение null),
            if (pathProgramFolder == null)
                //инициализируется новый экземпляр класса
                throw new InvalidOperationException("Call Initialize first");

            TFlex.ApplicationSessionSetup setup = new TFlex.ApplicationSessionSetup();

            //Режим "только для чтения" (если false - нужен ключ)
            setup.ReadOnly = false;

            setup.EnableMacros = true;

            return TFlex.Application.InitSession(setup);
        }

        //Деинициализация API
        static public void Terminate()
        {
            //Если дирректория с установленным T-FLEX CAD не существует (значение null),
            //то работа метода завершается
            if (pathProgramFolder == null)                
                return;
            
            //Завершение сеанса приложения
            TFlex.Application.ExitSession();

            //Обнуление значения переменной пути программной папки
            pathProgramFolder = null;
        }

        //Определение по записи в реестре пути до дирректории, в которую установлена программа
        static private string GetPath(string subkey_name)
        {
            //Чтение раздела HKEY_LOCAL_MACHINE "SOFTWARE\Top Systems" реестра Windows
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Top Systems", RegistryKeyPermissionCheck.ReadSubTree, System.Security.AccessControl.RegistryRights.ReadKey);
            
            //Если такого раздела не существует, то работа метода завершается с передачей параметра ""
            if (key == null)
                return "";

            //Создание текста, содержащего массив строк с именами всех вложенных разделов
            string[] top_systems_subkey_names = key.GetSubKeyNames();
            key.Close();

            //Создать список значений текущей версии и/или наименования программы
            List<string> subkey_names = new List<string>();

            //Цикл перебирающий имена разделов в тексте
            for (int i = 0; i < top_systems_subkey_names.Length; i++)
            {
                //Инициализация переменной версии и/или наименования программы
                string nameTFlexProgram = "";

                //Присвоение значения переменной в зависимости от версии и/или наименования программы
                if (subkey_name == "T-FLEX CAD 3D 11.0\\Rus")
                    nameTFlexProgram = "T-FLEX CAD";

                if (subkey_name == "T-FLEX Viewer 11.0\\Rus")
                    nameTFlexProgram = "T-FLEX Viewer";


                //Если в тексте, содержащего массив строк с именами всех вложенных разделов найдена запись
                //версии и/или наименования программы
                if (top_systems_subkey_names[i].ToString().Contains(nameTFlexProgram))
                {
                    //Присвоение переменной наименования раздела в реестре, соответствующего версии и/или наименованию программы
                    //HKEY_LOCAL_MACHINE\SOFTWARE\Top Systems\T-FLEX CAD 3D 11.0
                    key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Top Systems\" + top_systems_subkey_names[i], RegistryKeyPermissionCheck.ReadSubTree, System.Security.AccessControl.RegistryRights.ReadKey);

                    //Если раздела не существует - работа метода завершается с передачей параметра ""
                    if (key == null)
                        return "";

                    //Создание текста, содержащего массив строк с именами всех вложенных разделов
                    string[] tflex_cad_subkey_names = key.GetSubKeyNames();

                    //Цикл перебирающий имена разделов в тексте
                    for (int j = 0; j < tflex_cad_subkey_names.Length; j++)
                    {
                        //Присвоение переменной наименования папки в разделе реестра, соответствующего версии и/или наименованию программы
                        //SOFTWARE\Top Systems\T-FLEX CAD 3D 11.0\Rus
                        string str = string.Format(@"SOFTWARE\Top Systems\{0}\{1}", top_systems_subkey_names[i], tflex_cad_subkey_names[j]);
                        subkey_names.Add(str);
                    }

                    key.Close();
                }
            }

            //Если количество записей равняется нулю, то работа метода завершается с передачей параметра ""
            if (subkey_names.Count == 0)
                return "";

            //Иначе
            else
            {
                string regpath = "";
                if (subkey_name != null)
                {
                    for (int i = 0; i < subkey_names.Count; i++)
                    {
                        if (subkey_names[i].Contains(subkey_name))
                        {
                            //SOFTWARE\Top Systems\T-FLEX CAD 3D 11.0\Rus
                            regpath = subkey_names[i];
                            break;
                        }
                    }
                }

                else
                {
                    regpath = subkey_names[subkey_names.Count - 1];
                }
                if (regpath != "")
                {
                    //HKEY_LOCAL_MACHINE\SOFTWARE\Top Systems\T-FLEX CAD 3D 11.0\Rus
                    key = Registry.LocalMachine.OpenSubKey(regpath, RegistryKeyPermissionCheck.ReadSubTree, System.Security.AccessControl.RegistryRights.ReadKey);
                    
                    if (key == null)
                        return "";

                    string path = (string)key.GetValue("ProgramFolder", string.Empty);

                    //Получение пути дирректории, куда установлена программа
                    //например, C:\Program Files\T-FLEX\T-FLEX CAD 11\Program\
                    if (string.IsNullOrEmpty(path))
                        path = (string)key.GetValue("SetupHelpPath", string.Empty);
                    
                    key.Close();

                    if (path.Length > 0 && path[path.Length - 1] != '\\')
                        path += @"\";

                    return path;
                }
                return "";
            }
        }

        //Метод загружает содержимое библиотек, таких как: TFlexAPI.dll, TFlexMacroIDE.dll, TFlexMacroLoader.dll
        static private System.Reflection.Assembly AssemblyResolve(object sender, ResolveEventArgs args)
        {
            //Если дирректория с установленным T-FLEX CAD не существует,
            //то работа метода завершается
            if (pathProgramFolder == null || pathProgramFolder == "")
                return null;

            try
            {
                string name = args.Name;

                int index = name.IndexOf(",");
                if (index > 0)
                    name = name.Substring(0, index);

                string fileNameDLL = string.Format("{0}{1}.dll", pathProgramFolder, name);

                System.IO.Directory.SetCurrentDirectory(pathProgramFolder);

                return System.Reflection.Assembly.LoadFile(fileNameDLL);

            }
            catch (Exception ex)
            {
                return null;
            }
        }
    }
}
